{-----------------------------------------------------------------------------
 Unit Name: DUStrings
 Author:    Sebastian Htter
 Date:      2006-08-01
 Purpose:   String processing routines like Fetchtoke, Splitstring, div. Copy routines
            String comparison routines
 History:   2006-08-01 initial release
-----------------------------------------------------------------------------}
unit DUStrings;

interface

uses Sysutils;

type TStringArray = array of ShortString;

function  Cut   (var  s : string; index, count : Integer):string;
function CopyTo(S,Subst:string; Start:integer=1):string;
function CopyFrom(S,Subst:string; Count:integer=MaxInt):string;
function CopySubst(S,SubStart,SubEnd:string):string;

function SplitString (s:string; size:integer):TStringArray; overload;
function SplitString(s:string; delim:char):TStringArray;  overload;
function StrAGetVal(P:TStringArray; Name:string):string;
procedure StrAClear(var A:TStringArray);
procedure StrADelete(var A:TStringArray; Index:integer);
procedure StrAAdd(var A:TStringArray; S:String);
function ConcatString(parts:TStringArray):string;

function FetchToken(var s : string; delim : Char) : string;

function KillString(Substr,Str:String):String;

function QuotedString(const S: string; Quote: Char): string;
function ExtractQuotedString(const S: string; Quote: Char): string;

function RepeatString(S:String; Times:integer):String;
function RepeatChar(C:Char; Times:integer):String;
function StringFillRight(const S:String; Len:integer; Fill:char=' '):string;
procedure DeleteRight(var S:String; Num:integer);

function CompareText(S1,S2:String; IgnoreCase:boolean=true):integer;
function IsSameText(S1,S2:String; IgnoreCase:boolean=true):boolean;
function StartsWith(S1,S2:String; IgnoreCase:boolean=true):boolean;
function EndsWith(S1,S2:String; IgnoreCase:boolean=true):boolean;

procedure ConvertSpecials(var S:String);
// always with & an ;, eg. Str:= '&amp;'
function IsHTMLEntity(Str:String):boolean;

const
  Small_Letters = ['a'..'z'];
  Capitals      = ['A'..'Z'];
  Letters       = Small_Letters+Capitals;
  Numbers       = ['0'..'9'];
  AlphaNum      = Letters+Numbers;

implementation

function Cut   (var  s : string; index, count : Integer):string;
begin
  Result:= Copy(s,index,count);
  Delete(s,index,count);
end;

function CopyTo(S,Subst:string; Start:integer=1):string;
begin
  if Pos(Subst,S)>0 then
    Result:= copy(S,Start,pos(Subst,S)-Start)
  else
    Result:= S;
end;

function CopyFrom(S,Subst:string; Count:integer=MaxInt):string;
var i:integer;
begin
  i:= pos(Subst,S);
  Result:= copy(S,i+length(Subst),Count);
end;

function CopySubst(S,SubStart,SubEnd:string):string;
begin
  Result:= CopyFrom(S,SubStart);
  Result:= CopyTo(Result,SubEnd);
end;

function SplitString(s:string; size:integer):TStringArray;
var c:integer;
begin
  StrAClear(Result);
  c:= 0;
  while Length(s)> size do begin
    SetLength(result,c+1);
    Result[c]:= Cut(s,1,size);
    inc(c);
  end;
  SetLength(result,c+1);
  Result[c]:= Cut(s,1,size);
end;

function SplitString(s:string; delim:char):TStringArray;  overload;
var a:string;
begin
  StrAClear(Result);
  a:= fetchToken(s,delim);
  while a>'' do begin
    StrAAdd(Result,a);
    a:= fetchToken(s,delim);
  end;
end;

function StrAGetVal(P:TStringArray; Name:string):string;
var i:integer;
begin
  for i:= 0 to high(p) do
    if CopyTo(p[i],'=')=Name then begin Result:= CopyFrom(p[i],'='); exit end;
end;

procedure StrAClear(var A:TStringArray);
begin
  SetLength(A,0);
end;

procedure StrADelete(var A:TStringArray; Index:integer);
var I:integer;
begin
  for i:= index to High(A)-1 do
    A[i]:= A[i+1];
  SetLength(A,length(A)-1);
end;

procedure StrAAdd(var A:TStringArray; S:String);
begin
  SetLength(A,Length(A)+1);
  A[High(A)]:=S;
end;

function ConcatString(parts:TStringArray):string;
var i:integer;
begin
  Result:='';
  for i:= 0 to high(parts) do
    Result:= Result+parts[i];
end;

  // Fetch a token and delete it from string:
function FetchToken(var s : string; delim : Char) : string;
var i : integer;
    L : integer;
begin
   L := Length(s);
   Result := '';
   i := 1;
   while (i <= L) and not (s[i]=delim) do
   begin
     Result := Result + s[i];
     Inc(I);
   end;
   // skip delims
   while (i <= L) and (s[i] = delim) do
     Inc(i);
   s := Copy(s, i, L-i+1);
end;

function KillString(Substr,Str:String):String;
begin
  Result:= StringReplace(Str,Substr,'',[rfReplaceAll]);
end;

function QuotedString(const S: string; Quote: Char): string;
begin
  Result := AnsiQuotedStr(S, Quote);
end;

function ExtractQuotedString(const S: string; Quote: Char): string;
var
  P: PChar;
begin
  P := PChar(S);
  if P^ = Quote then
    Result := AnsiExtractQuotedStr(P, Quote)
  else
    Result := S;    
end;

function RepeatChar(C:Char; Times:integer):String;
begin
  SetLength(Result,Times);
  FillChar(Result[1],Times,C);
end;

function RepeatString(S:String; Times:integer):String;
var i:integer;
begin
  Result:= '';
  for i:=1 to Times do Result:= Result+S;
end;

function StringFillRight(const S:String; Len:integer; Fill:char=' '):string;
begin
  if Length(S)>=Len then
    Result:= S else
    Result:= S+RepeatChar(Fill,len-length(S));
end;

procedure DeleteRight(var S:String; Num:integer);
begin
  SetLength(S,length(S)-Num);
end;

function CompareText(S1,S2:String; IgnoreCase:boolean=true):integer;
begin
  if IgnoreCase then Result:= AnsiCompareText(S1,S2) else
    Result:= AnsiCompareStr(S1,S2);
end;

function IsSameText(S1,S2:String; IgnoreCase:boolean=true):boolean;
begin Result:= CompareText(S1,S2,IgnoreCase)=0; end;

function StartsWith(S1,S2:String; IgnoreCase:boolean=true):boolean;
begin
  Result:= IsSameText(copy(S1,1,length(S2)),S2,Ignorecase);
end;

function EndsWith(S1,S2:String; IgnoreCase:boolean=true):boolean;
begin
  Result:= IsSameText(copy(S1,length(S1)-length(S2)+1,length(S2)),S2,Ignorecase);
end;

procedure ConvertSpecials(var S:String);
begin
  s:= StringReplace(s,' /t ', #9,[rfReplaceAll]);
  s:= StringReplace(s,' /n ',#13,[rfReplaceAll]);
  s:= StringReplace(s,' // ','/',[rfReplaceAll]);
end;

function IsHTMLEntity(Str:String):boolean;
var i:integer;
begin
  Result:= (length(Str)>3) and // &lt; is 4
           (Str[1]='&') and (Str[length(str)]=';');
  if Result then
    for i:= 2 to length(Str)-1 do
      if not (Str[i] in AlphaNum) then
        begin Result:= false; exit end;
end;

end.
 
