{-----------------------------------------------------------------------------
 Unit Name: DUXMLParser
 Author:    Sebastian Htter
 Date:      01-08-2006
 Purpose:   Simple Parser for XML files, w/o MSXML

 History:   2006-08-01 Initial release
-----------------------------------------------------------------------------}
unit DUXMLParser;

interface

uses Classes, SysUtils, Contnrs;

const
  TagStart  = '<';
  TagEnd    = '>';
  TagClose  = '/';
  TagQuote  = '"';
  TagXMLDef = '?';
  AttrSep   = ' ';
  ValueSep  = '=';

  NodeDelim = '/';

type
  TXMLNode = class;

  TXMLElement = class
  private
    FName: string;
    FAttrs:TStringList;
    FText: string;
    FParent: TXMLElement;
    function GetItem(Name: string): TXMLNode;
    procedure SetItem(Name: string; const Value: TXMLNode);
    function GetItemCount: integer;
    function GetAttr(Name: string): string;
    procedure SetAttr(Name: string; const Value: string);
    function GetAttrCount: integer;
    procedure SetParent(const Value: TXMLElement);
    function GetLevel: integer;
    function GetItemAt(Index: Integer): TXMLNode;
    procedure SetItemAt(Index: Integer; const Value: TXMLNode);
  protected
    FItems:TObjectList;
    function FindItem(Name:string; var Item:TXMLNode):boolean;

    property Name:string read FName write FName;
    property TextData:string read FText write FText;
    property Level:integer read GetLevel;
    property AttrCount:integer read GetAttrCount;
    property Attributes[Name:string] : string read GetAttr write SetAttr;
  public
    constructor Create; virtual;
    destructor Destroy; override;

    property ParentNode:TXMLElement read FParent write SetParent;

    procedure LoadFromString(XML:String);

    property ItemCount:integer read GetItemCount;
    property Items[Name:string] : TXMLNode read GetItem write SetItem; default;
    property ItemAt[Index:Integer] : TXMLNode read GetItemAt write SetItemAt;

    function GetAttribute(Node, Attr: string): string;
    function GetText(Node: string): string;
    function GetNode(Node: string): TXMLNode;
  end;





  TXMLNode = class(TXMLElement)
  public
    property Level;
    property Name;
    property TextData;
    property AttrCount;
    property Attributes;
  end;


  TXMLFile = class(TXMLElement)
  private

  protected

  public
    procedure LoadFromFile(FileName:String);
    procedure LoadFromStream(Stream:TStream);
  end;



implementation

uses DUStrings, StrUtils;

{ TXMLElement }

constructor TXMLElement.Create;
begin
  inherited;
  FItems:= TObjectList.Create;
  FAttrs:= TStringList.Create;
end;

destructor TXMLElement.Destroy;
begin
  FItems.Free;
  FAttrs.Free;
  inherited;
end;

function TXMLElement.FindItem(Name: string; var Item: TXMLNode): boolean;
var i:integer;
begin
  for i:= 0 to ItemCount-1 do begin
    if IsSameText(name, TXMLElement(FItems[i]).Name) then begin
      Item:= TXMLNode(FItems[i]);
      Result:= true;
      exit;
    end;
  end;
  Item:= nil;
  Result:= false;
end;

function TXMLElement.GetItemCount: integer;
begin
  Result:= FItems.Count;
end;

function TXMLElement.GetItem(Name: string): TXMLNode;
begin
  FindItem(Name,Result);
end;

procedure TXMLElement.SetItem(Name: string; const Value: TXMLNode);
var i:integer;
begin
  for i:= 0 to ItemCount-1 do begin
    if IsSameText(name, TXMLElement(FItems[i]).Name) then begin
      FItems[i].Free;
      FItems[i]:= Value;
      exit;
    end;
  end;
end;

function TXMLElement.GetItemAt(Index: Integer): TXMLNode;
begin
  Result:= TXMLNode(FItems[Index]);
end;

procedure TXMLElement.SetItemAt(Index: Integer; const Value: TXMLNode);
begin
  FItems[Index]:= Value;
end;


procedure TXMLElement.LoadFromString(XML: String);
type
  TXMLState=(None,WaitName,WaitAttr,WaitValue,WaitValueQuoted);
const
  Spaces= [' ',#13,#10];
var i,p:integer;
    c:char;
    st:TxmlState;
    h:boolean;
    this,new: TXMLElement;
    astr:string;
begin
  FItems.Clear;
  i:= 1;
  st:= none;
  this:= self;
  while i<length(xml) do begin
    c:= xml[i];
    h:= false;
    if (st=None) and (c in spaces) then begin
      inc(i);continue;
    end;
    case c of
      TagStart : if St=None then begin
                   if (This.Name>'') and IsSameText(copy(xml,i+2,length(this.Name)),this.Name) then begin
                     st:= None;
                     h:= true;
                     inc(i,length(this.Name)+3);
                     this:= this.ParentNode;
                   end else begin
                     St:= WaitName;
                     h:= true;
                     new:= TXMLElement.Create;
                     new.ParentNode:= this;
                     this:= new;
                   end;
                 end;
      TagEnd  : if st in [WaitName,WaitAttr] then begin
                  st:= None;
                  h:= true;
                  p:= PosEx(TagStart+TagClose+this.Name+TagEnd,xml,i);
                  if (p>0) and (pos(TagStart,copy(xml,i+1,p-i-1))=0) then begin
                    this.FText:= copy(xml,i+1,p-i-1);
                    i:= p-1;
                  end;
                end;
      TagClose:if (st in [WaitName,WaitAttr]) and (xml[i+1]=TagEnd) then begin
                 h:= true;
                 this.FText:= trim(this.FText);
                 this:= this.ParentNode;
               end;
      TagXMLDef: if (st in [WaitName]) and (this.Name='') then begin
                  this.Name:= '?';
                  st:= WaitAttr;
                  h:= true;
                end else if (st in [WaitAttr]) and (this.Name='?') and (xml[i+1]=TagEnd) then begin
                 h:= true;
                 this.FText:= trim(this.FText);
                 this:= this.ParentNode;
               end;
      AttrSep:if St in [WaitAttr,WaitName,WaitValue] then begin
                h:= true;
                St:= WaitAttr;
                astr:= '';
              end;
    end;
    if not h then  begin
      case St of
        WaitName : this.Name:= this.Name+c;
        WaitAttr : if c=ValueSep then begin
                     st:= WaitValue; astr:= astr+'=';
                   end else astr:= astr+c;
        WaitValue: begin
                     astr:= astr+c;
                     if (c = TagQuote) and (CopyFrom(astr,ValueSep)=c) then
                       st:= WaitValueQuoted else
                     if c=AttrSep then begin
                       this.FAttrs.Add(astr);
                       st:= WaitAttr;
                     end;
                   end;
        WaitValueQuoted: begin
                           astr:= astr+c;
                           if c=TagQuote then begin
                             this.FAttrs.Add(astr);
                             st:= WaitAttr;
                           end;
                         end;
      end;
    end;
    inc(i);
  end;
end;

function TXMLElement.GetAttr(Name: string): string;
begin
  Result:= ExtractQuotedString(FAttrs.Values[Name],TagQuote);
end;

function TXMLElement.GetAttrCount: integer;
begin
  Result:= FAttrs.Count;
end;

procedure TXMLElement.SetAttr(Name: string; const Value: string);
begin
  FAttrs.Values[Name]:= QuotedString(Value,TagQuote);
end;



procedure TXMLElement.SetParent(const Value: TXMLElement);
begin
  if Assigned(FParent) then
    FParent.FItems.Remove(Self);
  FParent := Value;
  if Assigned(Value) then
    Value.FItems.Add(Self);
end;

function TXMLElement.GetLevel: integer;
var n:TXMLElement;
begin
  Result:= -1;
  n:= self;
  while assigned(n) do begin
    inc(Result);
    n:= n.ParentNode;
  end;
end;

function TXMLElement.GetText(Node: string): string;
var n:TXMLElement;
begin
  n:= GetNode(Node);
  if assigned(n) then Result:= n.TextData else Result:= '';
end;

function TXMLElement.GetAttribute(Node, Attr: string): string;
var n:TXMLElement;
begin
  n:= GetNode(Node);
  if assigned(n) then Result:= n.Attributes[Attr] else Result:= '';
end;

function TXMLElement.GetNode(Node: string): TXMLNode;
var p:TStringArray;
    i:integer;
    n:TXMLElement;
begin
  p:= SplitString(Node,NodeDelim);
  n:= self;
  for i:= 0 to high(p) do begin
    n:= n.Items[p[i]];
    if not Assigned(n) then break;
  end;
  if (i=length(p)) and (n <> self) then  // only self is not a TXMLNode
    Result:= TXMLNode(n)
  else Result:= nil;
end;

{ TXMLFile }

procedure TXMLFile.LoadFromFile(FileName: String);
var fs:TFileStream;
begin
  fs:= TFileStream.Create(FileName,fmOpenRead);
  try
    LoadFromStream(fs);
  finally
    fs.free;
  end;
end;

procedure TXMLFile.LoadFromStream(Stream: TStream);
var str:TStringStream;
begin
  str:= TStringStream.Create('');
  try
    str.CopyFrom(Stream,0);
    LoadFromString(str.DataString);
  finally
    str.Free;
  end;
end;

end.
