unit uLemmingsObjects;

interface

uses uLemmingsGame,FastGL, GLHelper, dglOpenGL, glBitmap, Geometry, Windows,
     ExtCtrls, uResources;

type


  TLemmingsActor = class(TLemmingsVisObject)
  private
    FMove: TVector3f;
    FLifeTime: DWORD;
    FBirthTime: DWORD;
    FLifeLength: DWORD;
  protected
    function StandOnGround:boolean;
    procedure DoMovement;
    procedure OnLifeStart; virtual;
    procedure OnLifeEnd;  virtual;
  public
    constructor Create(AWorld:TLemmingsWorld); override;
    property Movement : TVector3f read FMove write FMove;
    property LifeLength : DWORD read FLifeLength write FLifeLength;
    property BirthTime : DWORD read FBirthTime write FBirthTime;
    property LifeTime : DWORD read FLifeTime write FLifeTime;

    procedure Render; override;

    procedure LeftRight(X:TGLFloat);
    procedure Jump;
  end;

  TLemmingsWalker = class(TLemmingsActor)
  protected
    procedure OnLifeStart; override;
    procedure OnLifeEnd; override;
  end;


  TLemmingsDropDoor = class(TLemmingsVisObject)
    FTimer:TTimer;
  private
    FDropC: integer;
    FDropped: integer;
    function GetDropT: integer;
    procedure SetDropT(const Value: integer);
    procedure DropTimerEv(Sender: TObject);
  public
    constructor Create(AWorld:TLemmingsWorld); override;
    destructor Destroy; override;

    property DropCount:integer read FDropC write FDropC;
    property Dropped:integer read FDropped write FDropped;
    property DropTimer:integer read GetDropT write SetDropT;
  end;

implementation

uses Types;


{ TLemmingsActor }


constructor TLemmingsActor.Create(AWorld: TLemmingsWorld);
begin
  inherited;
  OnLifeStart;
end;

procedure TLemmingsActor.Jump;
begin
  if StandOnGround then FMove[1]:= -GravSpeed*15;
end;

procedure TLemmingsActor.LeftRight(X: TGLFloat);
begin
  FMove[0]:= FMove[0]+X;
end;

function TLemmingsActor.StandOnGround: boolean;
var bbr:TRect;
begin
  bbr:= WorldBBox;
  bbr:= TranslateRect(bbr,0,trunc(GravSpeed));
  Result:= World.Map.CheckForCollision(bbr)=crGoodField;
end;

procedure TLemmingsActor.DoMovement;
var bbr,bbw:TRect;
    i:integer;
    sog,solv:Boolean;
begin
  bbr:= WorldBBox;
  sog:= StandOnGround;

  if (FMove[0]<>0) and sog then begin
    FMove[1]:= 0;
    solv:= false;
    for i:= 0 to climbHeight do begin
      bbw:= TranslateRect(bbr,round(FMove[0]),-i);
      if World.Map.CheckForCollision(bbw)=crEmpty then begin
        FPos[1]:= FPos[1]-i;
        solv:=true;
        break;
      end;
    end;
    if not solv then FMove[0]:= -FMove[0];
  end;

  if sog then
    FPos[0]:= FPos[0]+FMove[0]
  else begin
    FMove:= VectorAdd(FMove,Gravity);
    FPos[1]:= FPos[1]+FMove[1];
  end;
end;

procedure TLemmingsActor.Render;
begin
  DoMovement;
  FLifeTime:= World.GetTimer-FBirthTime;
  if (FLifeLength>0) and (FLifeTime>FLifeLength) then
    OnLifeEnd;
  inherited;
end;

procedure TLemmingsActor.OnLifeStart;
begin
  FBirthTime:= World.GetTimer;
end;  

procedure TLemmingsActor.OnLifeEnd;
begin
  Free;
end;

{ TLemmingsDropDoor }

constructor TLemmingsDropDoor.Create(AWorld: TLemmingsWorld);
begin
  inherited;
  FTimer:= TTimer.Create(nil);
  FTimer.OnTimer:= DropTimerEv;
  FDropped:= 0;
  OpenTexture('resource\img\door.tga',50,25,Rect(0,0,100,50));
end;

destructor TLemmingsDropDoor.Destroy;
begin
  FTimer.Free;
  inherited;
end;

function TLemmingsDropDoor.GetDropT: integer;
begin
  Result:= FTimer.Interval;
end;

procedure TLemmingsDropDoor.SetDropT(const Value: integer);
begin
  FTimer.Interval:= Value;
end;

procedure TLemmingsDropDoor.DropTimerEv(Sender:TObject);
begin
  inc(FDropped);
  with TLemmingsWalker.Create(World) do begin
    Position:= Self.Position;
  end;

  if FDropped=FDropC then FTimer.Enabled:= false;
end;

{ TLemmingsWalker }

procedure TLemmingsWalker.OnLifeEnd;
begin
  inherited;
end;

procedure TLemmingsWalker.OnLifeStart;
begin
  inherited;
  OpenTexture('resource/img/actor1.tga',9,14,Rect(0,0,20,30));
  FMove[0]:= 4;
end;


end.
